/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 1998-1999 Netscape Communications Corporation. All
 * Rights Reserved.
 */

// This is where functions related to the quick reply box live.
const gSMimeContractID = "@mozilla.org/messenger-smime/smimejshelper;1";
const gISMimeJSHelper = Components.interfaces.nsISMimeJSHelper;

// interfaces
var nsIMsgCompDeliverMode = Components.interfaces.nsIMsgCompDeliverMode;
var nsIMsgCompSendFormat = Components.interfaces.nsIMsgCompSendFormat;
var nsIMsgCompConvertible = Components.interfaces.nsIMsgCompConvertible;
var nsIMsgCompType = Components.interfaces.nsIMsgCompType;
var nsIMsgCompFormat = Components.interfaces.nsIMsgCompFormat;
var nsIAbPreferMailFormat = Components.interfaces.nsIAbPreferMailFormat;
var nsIPlaintextEditorMail = Components.interfaces.nsIPlaintextEditor;

// static globals, need to be initialized only once
var sMsgComposeService = Components.classes["@mozilla.org/messengercompose;1"].getService(Components.interfaces.nsIMsgComposeService);

//added gAccountManager so that we can get the account in use -- needed for sendMsg - Frank DiLecce 
var gAccountManager = Components.classes["@mozilla.org/messenger/account-manager;1"].getService(Components.interfaces.nsIMsgAccountManager);
var gQRTemplbl; //a holder for the 'Show Quick Reply' button label text, held when we change the text to 'Hide Quick Reply' - Frank DiLecce
var gQRAltProgress; //an Alternate percent done Var - Frank DiLecce
var gPromptService;
var gMsgCompose;
var gSendNow;
var gSrcMsgURI;
var gTypedBdy;

var gQREditorWindow;
var msgCompose;
var sPrefs = null;  
var sPrefBranchInternal = null;
var sOther_headers = "";

var sAccountManagerDataSource = null;
var sRDF = null;
var sNameProperty = null;

var sQuickReplyMessagesBundle = document.getElementById("bundle_quickreplyMessages");
var sComposeMessagesBundle = document.getElementById("bundle_composeMessages");

try {
    var gDebugLevel;
	gDebugLevel = pref.getIntPref( 'quickreply.debug' );
} catch(e) {
    // If the debug level pref was unavailable,
    // can only assume no default prefs were set.. lets set them now - Frank DiLecce
    saveDefaultPrefs();
	gDebugLevel = 0;
}

// debug function.
// the higher level, the less chance it gets shown.
// the lower gDebugLevel, the less gets shown.
// debug levels:
//	5:	important information is logged, including catches.
//	10:	non-default paths are logged
// 	20:	all function starts get shown
//	25:	all key presses in text box are also shown.
function debug(level, message) {
    // attempting to change the way debug works so that you can change the level without restarting - Frank DiLecce
    try {
	   gDebugLevel = pref.getIntPref( 'quickreply.debug' );
    } catch(e) {
        // If the debug level pref was unavailable,
        // can only assume no default prefs were set.. lets set them now - Frank DiLecce
       saveDefaultPrefs();
	   gDebugLevel = 0;
    }
	if( level > gDebugLevel ) { return; }
    //moved this down here so that console would only be interrupted if in debugging mode - Frank DiLecce
    var sConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
	 sConsoleService.logStringMessage('QR: '+message);
}

// shortcut key for focus
function setFocusToQRTextInput(tgt) {
	debug(20, 'setFocusToQRTextInput: started');
    	//var viewQuickReplyToolbar;
    	//try {
        //	viewQuickReplyToolbar = pref.getBoolPref( 'quickreply.display_' + tgt );
    	//} catch (e) {
    	//	debug(5,'setFocusToQRTextInput: exception "'+e+'" caught when accessing preference quickreply.display_' + tgt);
    	//	viewQuickReplyToolbar = 1;
    	//}
    	//if(viewQuickReplyToolbar) {
    // no need to do the above since all states are QuickReply states are immediately available - Frank DiLecce
    var QRBox = document.getElementById('quickReplyInput')
    if (!QRBox.collapsed) QRBox.focus();
	   //}
}

// getActiveQuoteChoice
// depending on preferences
function getActiveQuoteChoice() {
	// quickreply.quote:
	//   0: Honour the account's quote preference
	//   1: Don't add a quote (default)
	//   2: Add the quote above 
	//   3: Add the quote below

    // returns :
    // 0 no quote
    // 1 reply above quote
    // 2 reply below quote

	debug(10, 'getActiveQuoteChoice: started');
	var shouldQuotePref, retval;
	// shouldQuotePref:
	// 0: use system pref, 1: don't, 2: above, 3: below
	try {
			shouldQuotePref = pref.getIntPref('quickreply.quoting');
	} catch (e) {
		debug(5,'getActiveQuoteChoice: exception "'+e+'" caught when accessing preference quickreply.quoting');
			shouldQuotePref = 2;
			pref.setIntPref( 'quickreply.quoting', shouldQuotePref );
	}
	
	if( shouldQuotePref == 0 ) {
	
		// mail.autoqoute defaults to true!
		retval = -1;
		try {
			// should we quote or not?
            //  changed the next line because quoting is account specific - Frank DiLecce
			if( false == pref.getBoolPref("mail.identity." + getCurrentIdentity().key + ".auto_quote") ) {
				retval = 0;
			} 
		} catch(e) {
			debug(5,'getActiveQuoteChoice: exception "'+e+'" caught when accessing preference mail.identity.<acctkey>.autoquote');
            // autoquote might not exist 
		}
		// we should quote, maybe. now, let's find out where.
		if( retval == -1 ) {
			try {
				// if the pref is there, it's either 1 (below) or 2 (select text)
            //  changed the next line because reply_on_top is account specific - Frank DiLecce
				switch( pref.getIntPref("mail.identity." + getCurrentIdentity().key + ".reply_on_top") ) {
					case 0: retval = 2; break;	// reply_on_top 0 = reply below quote
					case 1: retval = 1; break;	// reply_on_top 1 = reply above quote
					case 2: retval = 1; break;// select - interpret as above (we need to do something...)
					default: retval = 1;
				}
			} catch(e) {
				// if the getting fails, the pref is not there - and then we need to quote below.
				debug(5,'getActiveQuoteChoice: exception "'+e+'" caught when accessing preference mailnews.reply_on_top');
				retval = 0;
			}
		}
	} else {
		retval = shouldQuotePref -1;
	}
	return retval;
}

// getActiveSigChoice
// depending on preferences, 
function getActiveSigChoice( identity ) {
	debug(20, 'getActiveSigChoice: started');
	var retval, sigpref;
	// quickreply.signature:
	//   0: Honour the account's signature preference
	//   1: Don't add a signature (default)
	//   2: Add the signature at the bottom of the email
	//   3: Add the signature directly below my text

    // returns :
    // 0 no sig
    // 1 sig at bottom of mail
    // 2 sig at bottom of typed text
	try {
		sigpref = pref.getIntPref('quickreply.signature');
	} catch (e) {
		debug(5,'getActiveSigChoice: exception "'+e+'" caught when accessing preference quickreply.signature');
		sigpref = 1;
		pref.setBoolPref( 'quickreply.signature', sigpref );
	}
	if( sigpref == 0 ) {
		// if the pref says FALSE, don't add it (0)
		// if the pref says TRUE,  add it as follows:
		// if identity.sig_Bottom is not defined, then add at bottom (1)
		// if identity.sig_Bottom is defined and true, then add at bottom (1)
		// if identity.sig_Bottom is defined and false, then add directly below my text (2)
		if( identity.attachSignature ) {
			if( identity.sigBottom === undefined ) { // sigBottom is not defined - possibly have an old TB.
				retval = 0;
			} else {
                // sigbottom false = sig below the reply above quote if any
                // sigbottom true = reply on top, sig below quote 
				retval = ( identity.sigBottom ? 1 : 2 );
			}
		} else {
			retval = 0;
		}
	} else {
		retval = sigpref - 1; // what a coincidence...
	}
	return retval;
}

// menu item
function initQRMenuItem(tgt) {
	debug(20, 'initQRMenuItem: started');
    	//var viewQuickReplyToolbar;
    	//try {
    	//		viewQuickReplyToolbar = pref.getBoolPref('quickreply.display_' + tgt);
    	//} catch (e) {
    	//		debug(5,'initQRMenuItem: exception "'+e+'" caught when accessing preference quickreply.display_' + tgt);
    	//		viewQuickReplyToolbar = 1;
    	//		pref.setBoolPref( 'quickreply.display_' + tgt, viewQuickReplyToolbar );
    	//}
    // Above is not needed, now we just get the state of the QuickReply Box ad set the check accordingly - Frank DiLecce
    var viewQuickReplyToolbar = getViewQuickReplyToolbar(tgt);
	document.getElementById('menu_showQuickReplyToolbar').setAttribute("checked", viewQuickReplyToolbar ? "true" : "false");
}

function saveDefaultPrefs() {
    // this function would only run if the quickreply.debug pref does NOT exist
    //  this was added because I found that if the user did NOT go into settings right after installing Quick Reply
    //   it would not work - Frank DiLecce
	pref.setIntPref("quickreply.debug", 0);
	debug(20, 'saveDefaultPrefs: started');
	pref.setBoolPref("quickreply.deleteaftersend", true);
	pref.setBoolPref("quickreply.display_mailWindow", false);
    // The quickreply.ident pref is the account to use in Local Folders - Frank DiLecce
	pref.setIntPref("quickreply.ident", 1);
	pref.setIntPref("quickreply.quoting", 0);
	pref.setIntPref("quickreply.signature", 0);
}

function getViewQuickReplyToolbar(tgt) {
	debug(20, 'getViewQuickReplyToolbar: started');
    // the var below is now delclared, it wasn't before - Frank DiLecce
    var viewQuickReplyToolbar;
	try {
			viewQuickReplyToolbar = pref.getBoolPref( 'quickreply.display_' + tgt );
	} catch (e) {
			debug(5,'getViewQuickReplyToolbar: exception "'+e+'" caught when accessing preference quickreply.display_' + tgt );
    		viewQuickReplyToolbar = 0;
	}
    	//try {
    	//		viewQuickReplyToolbarToggle = pref.getBoolPref( 'quickreply.display_toggle_' + tgt );
    	//} catch (e) {
    	//		debug(5,'getViewQuickReplyToolbar: exception "'+e+'" caught when accessing preference quickreply.display_toggle_' + tgt );
    	//		viewQuickReplyToolbarToggle = 0;
    	//}
    	//if (viewQuickReplyToolbarToggle)
    	//	return true;
    //  Don't know what the toggle pref did, seemed to do the same thing as QuickReplyToolbar pref, very confusing
    //   disabled its functionality and deleted all references to it - Frank DiLecce
	return viewQuickReplyToolbar;
}

function setViewQuickReplyToolbar(tgt,viewQuickReplyToolbar) {
	debug(20, 'setViewQuickReplyToolbar: started');
    // all this function should do is set the pref and menu item checkmark - Frank DiLecce
	if(viewQuickReplyToolbar) {
			document.getElementById('menu_showQuickReplyToolbar').setAttribute("checked", "true");
            // commented next line, it's now done elsewhere - Frank DiLecce
			//ShowQuickReplyBox(tgt);
	} else {
			document.getElementById('menu_showQuickReplyToolbar').setAttribute("checked", "false");
            // commented next line, it's now done elsewhere - Frank DiLecce
			//HideQuickReplyBox(tgt);
	}
    // commented next line, toggle functionality removed - Frank DiLecce
	//pref.setBoolPref( 'quickreply.display_toggle_' + tgt, viewQuickReplyToolbar == 2 );
	pref.setBoolPref( 'quickreply.display_' + tgt, viewQuickReplyToolbar );
}

function toggleQuickReplyMenuItem(tgt) {
	debug(20, 'toggleQuickReplyMenuItem: started');
    // this function now just hands off to toggleQuickReplyButton because it should all be done in one place - Frank DiLecce
    toggleQuickReplyButton(tgt);
	//var viewQuickReplyToolbar = getViewQuickReplyToolbar(tgt);
	//if (viewQuickReplyToolbar == 0)
	//	viewQuickReplyToolbar = 1;
	//else
	//	viewQuickReplyToolbar = 0;
	//setViewQuickReplyToolbar(tgt, viewQuickReplyToolbar);
}

function toggleQuickReplyButton(tgt) {
	debug(20, 'toggleQuickReplyButton: started');
    // we are actually gonna use the button label to set and retrieve QuickView Box state - Frank DiLecce
	   //var viewQuickReplyToolbar = getViewQuickReplyToolbar(tgt);  
    var QRtoggleButton = document.getElementById("quickReplyToggleButton");   // the button on extendedview
    var QRtoggleHidden = document.getElementById("quickReplyHiddenText");     // hidden text label contains the opposite of "Show Quick View"
    if (QRtoggleButton.label != QRtoggleHidden.value) {
        gQRTemplbl = QRtoggleButton.label
        QRtoggleButton.label = QRtoggleHidden.value;
        ShowQuickReplyBox(tgt);
    } else {
        QRtoggleButton.label = gQRTemplbl
        HideQuickReplyBox(tgt);
    }
	//if (viewQuickReplyToolbar == 0)
	//	viewQuickReplyToolbar = 2;
	//else
	//	viewQuickReplyToolbar = 0;
	//setViewQuickReplyToolbar(tgt, viewQuickReplyToolbar);
}

function ShowQuickReplyBox(tgt) {
	debug(20, 'ShowQuickReplyBox: started');
	var QRBox = document.getElementById( 'msgQuickReply_' + tgt);
	if (QRBox) {
		QRBox.collapsed = false;
    // we set the pref from here now - Frank DiLecce
        setViewQuickReplyToolbar(tgt,true);
    }
} 

function HideQuickReplyBox(tgt) { 
	debug(20, 'HideQuickReplyBox: started');
	var QRBox = document.getElementById('msgQuickReply_' + tgt);
	if (QRBox) {
        QRBox.collapsed = true;
        setViewQuickReplyToolbar(tgt,false);
        // lets clear the QuickReply textbox here so user does't get surprised later on when he shows it again - Frank DiLecce
        document.getElementById("quickReplyInput").value="";
    }
}

function HideQRBox(tgt) { 
// This function gets called from an observer in the xul
// this function hides the QuickReply Box if the extendedMessageHeader is collapsed 
//         (we don't really want to show it if there is no message selected) 
//  This function shows The Quick Reply box IF the Quick Reply button says 'Hide Quick Reply' 
//   this would happen when the user switches folders
	debug(20, 'HideQRBox: started');
	var QRBox = document.getElementById('msgQuickReply_' + tgt);
    var QRtoggleButton = document.getElementById("quickReplyToggleButton");
    var QRtoggleHidden = document.getElementById("quickReplyHiddenText");
    var ExpHeaderView = document.getElementById("expandedHeaderView");    
    if (ExpHeaderView.collapsed) {
        // we don't want the Quick Reply Box to be shown if there is no message shown
    	if (QRBox) QRBox.collapsed = true;
    } else {
        if (QRtoggleButton.label == QRtoggleHidden.value) {
            ShowQuickReplyBox(tgt);
        } else {
            HideQuickReplyBox(tgt);
        }
    }
}


function getCurrentIdentity() {
	debug(20, 'getCurrentIdentity: started');
	var server = GetLoadedMsgFolder().server;
	var identity = accountManager.getFirstIdentityForServer( server );
	if( ! identity ) {
        // if identity is blank, lets use the one from prefs - Frank DiLecce
        identity = accountManager.getIdentity("id" + pref.getIntPref( 'quickreply.ident' ));
		debug(10, 'Default identity was blank, using pref Identity');
    }
	debug(10, 'Identity: '+identity);
	return identity;
}


function setStatus(newStatus) {
	document.getElementById("statusText").setAttribute("label", newStatus);
}


function ShowPref(tgt) {
// This function shows the options panel - Frank DiLecce ( added comment only )
	debug(20, 'ShowPref: started');
	window.openDialog('chrome://messenger-quickreply/content/prefQuickReply.xul', 'messenger-quickreply-pref', 'chrome');
}

function disableQR() {
// this function is called in the doReply function - Frank DiLecce ( added comment only )
	debug(20, 'disableQR: started');
	document.getElementById("sendButton").disabled=true;
	document.getElementById("quickReplyInput").disabled=true;
}

function enableQR() {
// this function is called in the doReply function - Frank DiLecce ( added comment only )
	debug(20, 'enableQR: started');
	document.getElementById("sendButton").disabled=false;
	document.getElementById("quickReplyInput").disabled=false;
}

function clearQR() {
// this function is called in the doReply function if the option for it is ticked - Frank DiLecce ( added comment only )
	debug(20, 'clearQR: started');
	try {
		if( pref.getBoolPref( 'quickreply.deleteaftersend') )
			document.getElementById("quickReplyInput").value="";
	} catch(e) {
		debug(5,'clearQR: exception "'+e+'" caught when accessing preference quickreply.deleteaftersend');
		document.getElementById("quickReplyInput").value="";
	}
}
function doQRTextBoxFocus(e) {
	debug(20, 'doQRTextBoxFocus: started');
//	var tb = document.getElementById('quickReplyInput');
	setStatus('tb focus '+e.type);
}

// Killed out the whole of the next function since it was all commented anyway and was not called from anywhere
//function doQRTextBoxClick(e) {
//	debug(20, 'doQRTextBoxClick: started');
//	var tb = document.getElementById('quickReplyInput');
//	if(!tb.focussed) tb.select();
//}

function doQRTextBoxKeyPress(e, tgt) {
	debug(25, 'doQRTextBoxKeyPress: '+e.type+"|ctrl:"+e.ctrlKey+"|alt:"+e.altKey+"|keyCode:"+e.keyCode+"|charCode:"+e.charCode+"|which:"+e.which);
	if(e.ctrlKey == true && e.altKey == false && e.metaKey == false && e.which== 13 ) { // Enter: 13
		// ctrl+enter --> send; check if that's ok.
		disableQR();
		if( !maybeWarnOnSend() ) {
			enableQR();
			return;
		}
		doQuickReply(e, -1, tgt);	// just pass the same event - saves the hassle.
	} else if(e.ctrlKey == false && e.altKey == false && e.metaKey == false && e.keyCode == 9) {
		// tab: insert tab character.
		e.preventDefault();
		var textbox = document.getElementById("quickReplyInput")
		var content = textbox.value;
		var selStart = textbox.selectionStart;
		var selEnd = textbox.selectionEnd;
		textbox.value = content.substr(0, selStart )+"\t"+content.substr(selEnd, content.length );
		textbox.setSelectionRange( selStart+1, selStart+1);
	}
}

//	This returns the original message (or a selected part) quoted for inclusion in the reply
function getQuotedOriginal( author ) {
	debug(20, 'getQuotedOriginal: started');
	// try to get selection from the message pane
	var composeMsgsBundle = document.getElementById("bundle_composeMessages");
	
    // modified next line because GetMessagePaneFrame was not accessable from here
    // GetMessagePaneFrame is a one line function -> return window.content - Frank Dilecce
	//var messagepaneframe = GetMessagePaneFrame();
	var messagepaneframe = window.content;
	var selection = messagepaneframe.getSelection();
	var selectiontext = selection.toString();
	if (selectiontext == "") {
		// if no selection, quote the whole document...
		selection.selectAllChildren(messagepaneframe.document.body);
		selectiontext = selection.toString();
		selection.collapseToStart();
	}
	// say who wrote the quoted part
	var authorwrote;
	try {
		authorwrote = composeMsgsBundle.getString('mailnews.reply_header_authorwrote')
	} catch(e) {
		debug(5,'getQuotedOriginal: exception "'+e+'" caught when accessing message mailnews.reply_header_authorwrote' );
		authorwrote = '';
	}

	// var quoteauthor = author + " wrote:";
	var quoteauthor;
	if( authorwrote == '' ) {
		quoteauthor = author;
	} else {
	 	quoteauthor = authorwrote.replace( '%s', author ) + ':';
	}

	// Add an indentCharacter at the beginning of each line.
	// The indentCharacter should be localised in the main app, but I can't find where - pending that, it's in our
	// personal properties. The indentCharacter defaults to '> '.
	var indentCharacter;
	try {
		/*@*/
		indentCharacter = document.getElementById("bundle_quickreplyMessages").getString('indentCharacter');
	} catch (e) {
		debug(5,'getViewQuickReplyToolbar: exception "'+e+'" caught when getting string indentCharacter' );
		indentCharacter = '> ';
	}

        var quotedsection = "";
        var eolpos = 0;
        var endofline = new RegExp("$", "m");
        var stilltoreplace = selectiontext;
        while (stilltoreplace != "") {
		eolpos = stilltoreplace.search(endofline);
		if (eolpos >= 0) {
			quotedsection = quotedsection + indentCharacter + stilltoreplace.slice(0, eolpos+1);
			stilltoreplace = stilltoreplace.slice(eolpos+2);//1);
		} else {
			quotedsection = quotedsection + indentCharacter + stilltoreplace;
		}
        }
	// put them all together...
	return quoteauthor + "\n" + quotedsection + "\n";
}

// function doQuickReply
//	This is the callback of the QuickReply button.
//	e: event that caused this click.
//	quote: if the original message should be quoted or not. -1 = use default value, 0 = don't, 1=above, 2=below
function doQuickReply(e, quote, tgt) {
	debug(10, 'doQuickReply: started');
	if (document.getElementById("quickReplyInput").value == null ||
			document.getElementById("quickReplyInput").value == "")	{
            // modified the grammer of next line just because with is correct (not to) - Frank DiLecce
			//debug(5, 'doQuickReply: nothing to reply to - returning');
			debug(5, 'doQuickReply: nothing to reply with - returning');
			return;
	}	
	var srcMsgHdr, dest, newsgroups, author, bdy, typedbdy, identity;
	disableQR();
	var srcMsgURI = GetLoadedMessage();
	try {
			srcMsgHdr = messenger.messageServiceFromURI(srcMsgURI).messageURIToMsgHdr(srcMsgURI)//GetMsgHdrFromUri(srcMsgURI);
	} catch (ex) {
			debug(5,'doQuickReply: exception "'+e+'" caught when trying to get message header for URI '+srcMsgURI+'.' );
			enableQR();
			return;
	}
	var srcMsgId = srcMsgHdr.messageId;
	var subj = srcMsgHdr.subject; //currentHeaderData["subject"].headerValue;

	// dwlt: is this localized in the main Mozilla app? If so, is there a way to request this value?
	//		::can't find it in the main components anywhere, but haven't looked in the C++ source base
	//		::if we can't find it there, we should just put in the quickreply.properties file
	
	// replySubjectPrefix is normally 'Re: '
	var replySubjectPrefix;
	try {
		/*@*/
		replySubjectPrefix = document.getElementById("bundle_quickreplyMessages").getString('replySubjectPrefix');
	} catch (e) {
		debug(5,'doQuickReply: exception "'+e+'" caught when getting string replySubjectPrefix' );
		replySubjectPrefix = 'Re: ';
	}
	debug(15,'doQuickReply: replySubjectPrefix: "'+replySubjectPrefix+'".' );
	
	if( subj.substring(0,4).toLowerCase() != replySubjectPrefix.toLowerCase() ) {
			subj = replySubjectPrefix + subj;
	}

	dest = "";
	newsgroups = "";
	if ('newsgroups' in currentHeaderData) {
		if ('followupto' in currentHeaderData)
			newsgroups = currentHeaderData['followupto'].headerValue;
		else
			newsgroups = currentHeaderData['newsgroups'].headerValue;
		debug(10, 'doQuickReply: replying to news groups: '+newsgroups);
	} else {
		if ('reply-to' in currentHeaderData)
			dest = currentHeaderData['reply-to'].headerValue;
	else
			dest = srcMsgHdr.author;
		debug(10, 'doQuickReply: replying to email: '+dest);
	}
	
	var identity = getCurrentIdentity();
	if( ! identity ) {
		doReplyFailed( document
					.getElementById("bundle_quickreplyMessages")
					.getString('failCurrentIdentity'),
					'' );
	}
	// This shouldsig block needs a little TLC
    // With textfile sigs whose last character is a letter, that last letter does not get included in the message !
    //  Was not sufficiently bothered to fix it - Frank DiLecce (sigs should end with a carrage return anyway right ?)
/*
	var shouldSig = getActiveSigChoice( identity );
	debug(15,'doQuickReply: shouldSig: '+shouldSig+'.' );
	var sigContent = '';
	if( shouldSig > 0 ) {
		var sigFile = identity.signature;
		if (sigFile) {
			try{
				//var obj_File = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
				//obj_File.initWithPath(str_Filename);
	
				var sigInputStream =  Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
				sigInputStream.init( sigFile, 0x01, 0444, null );
				
				var sigScriptableIO = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
				sigScriptableIO.init(sigInputStream);
			} catch (e) {
				debug(5,'doQuickReply: exception "'+e+'" caught when accessing sig file "'+sigFile.persistentDescriptor+'". Does the file exist?');
			}

			try {
				sigContent = sigScriptableIO.read( sigFile.fileSize - 1);
			} catch (e) {
				sigContent = '';
				debug(5,'doQuickReply: exception "'+e+'" caught when reading sig file "'+sigFile.persistentDescriptor+'". Does the file exist?');
			}

			try {
			sigScriptableIO.close();
			sigInputStream.close();
			} catch (e) { //don't report - not of consequence, really.
			}
		} else {
			debug(5,'doQuickReply: signature required but not defined for identity '+identity);
		}
	}

*/
    var shouldSig = 0; 
	var shouldQuote, defQuotePref = 0;	
	bdy = document.getElementById("quickReplyInput").value;
	typedbdy = bdy;
	// do we need to quote?
    // lets just get the value from the getActiveQuoteChoice function - Frank Dilecce
	// if( quote != 0 ) {
	if( 0 == 0 ) {
		// use the standard preference
        // lets just get the value from the getActiveQuoteChoice function - Frank Dilecce
		// if( quote < 0 ) {
		if( 0 == 0 ) {
			defQuotePref = getActiveQuoteChoice();
		} else {
			switch( quote ) {
				// 1: above, 2: below
				case 1: case 2: defQuotePref = quote; break;
				default: defQuotePref = 0;
			}
		}
		debug(15,'doQuickReply: defQuotePref: "'+defQuotePref+'".' );
		// add sig below - so add to bdy - no matter what happens,
		//  it'll be in the right spot
		if ( shouldSig == 2 ) {
			bdy = bdy + "\n\n-- \n" + sigContent;
		}
		if( defQuotePref > 0 ) {
			var orig = getQuotedOriginal(srcMsgHdr.author);
			if( defQuotePref == 1 ) {
                // we must no have the -- in the body if the quote is below the sig - Frank DiLecce
        		if ( shouldSig == 2 ) {
        			bdy = typedbdy + "\n\n" + sigContent;
        		}                
				bdy = bdy + "\n\n" + orig;
			} else {
				bdy = orig + "\n\n" + bdy;
			}
		}
		// add sig at the bottom.
		if( shouldSig == 1 ) {
			bdy += "\n\n-- \n" + sigContent;
		}
	} else {
		// don't care about whether it's bottom or below - amounts to same anyway.
		if( shouldSig > 0 ) {
			bdy = bdy + "\n\n-- \n" + sigContent;
		}
	}
    // added a linefeed to the body no matter what.. sendreply fails (on some occasions) if missing - Frank DiLecce
    bdy += "\n";
	doReply( e.shiftKey == false, dest, newsgroups, subj, typedbdy, srcMsgId, srcMsgURI, bdy, identity );
	// just gonna hide the Quick Reply Box on send so the use knows something happened
	toggleQuickReplyButton(tgt);
}

// Check mail.warn_on_send_accel_key, and if necessary, ask for confirmation.
// Returns true if it's OK to send, false if not.
function maybeWarnOnSend() {
	debug(20, 'maybeWarnOnSend: started');
    var warn = pref.getBoolPref("mail.warn_on_send_accel_key");
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    var composeMsgsBundle = document.getElementById("bundle_composeMessages");

    if (warn) {
        var checkValue = {value:false};
        var buttonPressed = promptService.confirmEx(window, 
              composeMsgsBundle.getString('sendMessageCheckWindowTitle'), 
              composeMsgsBundle.getString('sendMessageCheckLabel'),
              (promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_0) +
              (promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_1),
              composeMsgsBundle.getString('sendMessageCheckSendButtonLabel'),
              null, null,
              composeMsgsBundle.getString('CheckMsg'), 
              checkValue);
        if (buttonPressed != 0) {
        	debug(10, 'maybeWarnOnSend: sending cancelled.');
            return false;
        }
        if (checkValue.value) {
            pref.setBoolPref("mail.warn_on_send_accel_key", false);
        }
    }
    return true;
}

// Under-the-hood stuff
function onSMIMEStartHeadersQuickReply() {
}

function onSMIMEEndHeadersQuickReply() {
	ShowQuickReplyBox();
    //next line would throw an error because the element does not exist ( *I* can't find it at least) - Frank DiLecce
	//document.getElementById("QuickReplyTarget").value = currentHeaderData["to"].headerValue ;
}

function regListeners() {
	var listener = {};
	listener.onStartHeaders = onSMIMEStartHeadersQuickReply;
	listener.onEndHeaders = onSMIMEEndHeadersQuickReply;
	gMessageListeners.push(listener);
}

function QRMessagePaneOnLoad() {
// window.setTimeout("regListeners()", 1000);
    // added next 10 lines so that the state of the Quick Reply box persists correctly when moving amongst folders
    // called from listener below this function - Frank DiLecce
	debug(20, 'QRMessagePaneOnLoad: started');
    if (getViewQuickReplyToolbar('mailWindow')) {
        // if QuickRelpyToolbar is set to true we want the Quick Reply Button to show the correct value 
        //     which in this case would be "Hide Quick Reply"
    	debug(20, 'QRMessagePaneOnLoad: QuickReply pref was true');
        var QRtoggleButton = document.getElementById("quickReplyToggleButton");
        var QRtoggleHidden = document.getElementById("quickReplyHiddenText");
        gQRTemplbl = QRtoggleButton.label
        QRtoggleButton.label = QRtoggleHidden.value;
    } 
	regListeners();
}

function QRMessagePaneOnUnload() {
    // called from listener below this function - Frank DiLecce
	debug(20, 'QRMessagePaneOnUnLoad: started');
}

function onComposerSendMessage()
// added to see if this is making any difference at all - Frank Dilecce
{
  debug(20, 'onComposerSendMessage: started');
  try {
    if (!msgCompose.compFields.securityInfo.requireEncryptMessage) {
        debug(20, 'onComposerSendMessage: requireEncrypt is false');
      return;
    }
    debug(20, 'onComposerSendMessage: requireEncrypt is true, send might fail');

    var helper = Components.classes[gSMimeContractID].createInstance(gISMimeJSHelper);

    var emailAddresses = new Object();
    var missingCount = new Object();

    helper.getNoCertAddresses(msgCompose.compFields, missingCount, emailAddresses);
  }
  catch (e)
  {
    debug(20, 'onComposerSendMessage: in first catch, must not have securityInfo');

    return;
  }
  debug(20, 'onComposerSendMessage: helper is set');


  if (missingCount.value > 0)
  {
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefs = prefService.getBranch(null);

    var autocompleteLdap = false;
    autocompleteLdap = prefs.getBoolPref("ldap_2.autoComplete.useDirectory");

    if (autocompleteLdap)
    {
      var autocompleteDirectory = null;
      autocompleteDirectory = prefs.getCharPref("ldap_2.autoComplete.directoryServer");

      if(getCurrentIdentity().overrideGlobalPref) {
        autocompleteDirectory = getCurrentIdentity().directoryServer;
      }

      if (autocompleteDirectory)
      {
        window.openDialog('chrome://messenger-smime/content/certFetchingStatus.xul',
          '',
          'chrome,resizable=1,modal=1,dialog=1', 
          autocompleteDirectory,
          emailAddresses.value
        );
      }
    }
  }
  debug(20, 'onComposerSendMessage: ending');

}

addEventListener("messagepane-loaded", QRMessagePaneOnLoad, true);
addEventListener("messagepane-unloaded", QRMessagePaneOnUnload, true);
//addEventListener('compose-send-message', onComposerSendMessage, true);

// The new, nice, simple way of getting notified when a new editor has been created
var gMsgEditorCreationObserver =
{ 
  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic == "obs_documentCreated")
    {
      debug(20, 'gMsgEditorCreationObserver: getting Current Editor');
      var editor = GetCurrentEditor();
      if (editor)
        msgCompose.initEditor(editor, window.content);
        DoSend();
    }
  }
}

function markMessageReplied( msgURI) {
	debug(20, 'markMessageReplied: started');
	var msgHdr = messenger.messageServiceFromURI(msgURI).messageURIToMsgHdr(msgURI)//GetMsgHdrFromUri(srcMsgURI);
	if( !msgHdr) {
		return;
	}
	var msgFolder = msgHdr.folder;
	if (msgFolder) {
		msgFolder.addMessageDispositionState(msgHdr, 0);//msgFolder.nsMsgDispositionState_Replied
	}
}

function doReplyFailed( why, content ) {
	debug(5, 'Quick Reply failed: '+why);

	if(why) { dump(why); }
	setStatus( why );
	setTimeout( "setStatus('')", 10000 );
	if( content.length > 0 ) {
		document.getElementById("quickReplyInput").value = content;
	}
	enableQR();
	return false;
}


function doReply( sendNow, to, newsgroups, sbj, typedbdy, srcId, srcMsgURI, bdy, identity ) {
	debug(10, 'doReply: started');

	var compfields = Components.classes["@mozilla.org/messengercompose/composefields;1"].createInstance(Components.interfaces.nsIMsgCompFields);
	if(!compfields) {
		return doReplyFailed(document
				.getElementById("bundle_quickreplyMessages")
				.getString('failCompFields'),
				typedbdy);
				//"Failed to initialise compfields for quick reply."
	}
	msgCompose = Components.classes["@mozilla.org/messengercompose/compose;1"].createInstance(Components.interfaces.nsIMsgCompose);
	if(!msgCompose) {
		return doReplyFailed(document.getElementById("bundle_quickreplyMessages").getString('failMsgCompose') );//"Failed to initialise compfields for quick reply."
	}
	var params = Components.classes["@mozilla.org/messengercompose/composeparams;1"].createInstance(Components.interfaces.nsIMsgComposeParams);
	if(!params) {
		return doReplyFailed(sQuickReplyMessagesBundle.getString('failInitParams') ); //"Failed to initialise params for quick reply."
			return doReplyFailed(document
					.getElementById("bundle_quickreplyMessages")
					.getString('failInitParams'),
					typedbdy);
					//"Failed to initialise params for quick reply."
	}

    if (to != "")
    compfields.to = to;
    if (newsgroups != "") {
        compfields.newsgroups = newsgroups;
        debug(15, 'doReply: newsgroups: '+newsgroups); 
    }
    compfields.subject = sbj;
    compfields.body = bdy ;
    compfields.references = "<"+srcId+">";
    debug(15, 'doReply: to: '+to);
    
    params.identity = identity;
    params.composeFields = compfields;
    //params.format = 2;	// PlainText, otherwise we loose our formatting!
    params.format = 2;	
    
    
// new stuff    
    // The window below is made because the mozilla sendmsg function closes the nominated window upon completion
    gQREditorWindow = window.open("chrome://messenger-quickreply/content/QREditor.xul", "_blank", "chrome,extrachrome,resizable");
    // get the editor element.  this element is actually in the msgQuickReplyOverlay.xul -Frank DiLecce
    var editorElement = GetCurrentEditorElement();
    var editortype = "textmail";
    // make it editable
    editorElement.makeEditable("textmail", true);
    // fill it with something
    editorElement.webNavigation.loadURI("about:blank", // uri string
                           0,                            // load flags
                           null,                         // referrer
                           null,                         // post-data stream
                           null);
    
    // initialized msgCompose and nominate QREditorWindow as the window to close
    msgCompose.Initialize(gQREditorWindow, params );
    // set the global gSendNow var
    gSendNow = sendNow;
    // set the global gSrcMsgURI
    gSrcMsgURI = srcMsgURI;
    // set the global gTypedBdy
    gTypedBdy = typedbdy;
    // start the command observer to wait for the editor to be initialized
    // need to do this or text is not checked for character set conversion
    debug(5, 'doReply: starting command observer');
    editorElement.commandManager.addCommandObserver(gMsgEditorCreationObserver, "obs_documentCreated");
    debug(5, 'doReply: ending');

}

function DoSend() {
// Things in this function that must be done regardless of sending success are :
// set msgCompose to null
// unregister progresslistener
// remove the editorelement's command observer
// clear the editor - haven't figured this out yet, for now it gets overwritten ?
// the temporary window *Should* get closed by the mozilla sendMsg function but we will do it in the catch at the end
//   of this function

debug(5, 'doSend: starting');

    var identity = getCurrentIdentity()
    // added next two vars for the progress bar - Frank DiLecce
    var QRStatusBar = document.getElementById("statusbar-progresspanel");
    var QRStatusBarIcon = document.getElementById("statusbar-icon");

    var progressListener = {
		srcMessageId: String,
        onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus)
        {
          if (aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_START)
          {
				setStatus( document.getElementById("bundle_quickreplyMessages").getString('statusStartingSend') );
                // init the progress bar and altprogress vars - Frank DiLecce
                QRStatusBar.collapsed = false
                QRStatusBarIcon.setAttribute( "mode", "normal" );
                gQRAltProgress = 0;
          }
          
          if (aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_STOP)
          {
            // we are done sending/saving the message...
            // Indicate completion in status area.
    
				clearQR();
				enableQR();
				setStatus( document.getElementById("bundle_quickreplyMessages").getString('statusMessageSent') );
                QRStatusBarIcon.setAttribute("mode","normal");
                // Need to put everything to do with progress bar back to normal - Frank DiLecce
                QRStatusBarIcon.value = 0;  
                QRStatusBarIcon.label = "";
                QRStatusBar.collapsed = true;
				window.setTimeout("setStatus('')", "3000");
				markMessageReplied( this.srcMsgURI );
				debug(5, 'onStateChange: message sent');
                gPromptService = null;
                gMsgCompose = null;
                msgCompose = null;
            	progress.unregisterListener(progressListener);    
                GetCurrentEditorElement().commandManager.removeCommandObserver(gMsgEditorCreationObserver, "obs_documentCreated");
                GetCurrentEditorElement().makeEditable("textmail", false);
          }
        },
        
        onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress)
        {
          // Changed the whole ProgressChange Function to match TB's own (to a point) - Frank DiLecce
          // Calculate percentage.
          var percent;
		  debug(10, 'onStatusChange: '+aCurSelfProgress + " " +  aMaxSelfProgress + " " +  aCurTotalProgress + " " + aMaxTotalProgress);
          if ( aMaxTotalProgress > 0 ) 
          {
            percent = Math.round( (aCurTotalProgress*100)/aMaxTotalProgress );
            if ( percent > 100 )
              percent = 100;
            
			setStatus( document.getElementById("bundle_quickreplyMessages").getString('statusMessageSending') + '  ' + percent + '% fd' );
            // this sets the progress bar value - Frank DiLecce
            QRStatusBar.collapsed = false;
            QRStatusBarIcon.value = percent; 
            QRStatusBarIcon.label = percent + "%"
          } 
          else 
          {
            // Progress meter should be barber-pole in this case.
            // yea right! 
            // aMaxTotalProgress is always -1 for some reason and the aCurTotalProgress starts at 20 something
            //  and goes up to 1300 something ??  
            //  but it goes from say 150 to 30 to 180 to 50 etc.  so we save the highest value and only adjust if
            //  an incoming value is greater than that - Frank DiLecce 
            // I'm just gonna divide by 10 and max out at 100
            if (aCurTotalProgress > gQRAltProgress) {
                gQRAltProgress = aCurTotalProgress ;
                percent = Math.round(aCurTotalProgress/10);
                if ( percent > 100 )
                  percent = 100;
    			setStatus( document.getElementById("bundle_quickreplyMessages").getString('statusMessageSending') + '  ' + percent + '%' );
                QRStatusBar.collapsed = false;
                QRStatusBarIcon.value = percent; 
                QRStatusBarIcon.label = percent + "%"
                debug(10, 'onStatusChange:max is -1 percent is '+percent+ ' statbar is ' + document.getElementById("statusbar-progresspanel").getAttribute( "mode"));
            }              
          }
        },
    
    	onLocationChange: function(aWebProgress, aRequest, aLocation)
        {
          // we can ignore this notification
        },
    
        onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage)
		{	// Looks like it's possible that we get call while the document has been already deleted!
			// therefore we need to protect ourself by using try/catch
			debug(10, 'onStatusChange: '+aStatus+':'+aMessage+'.');
			try {
				setStatus( aMessage );
				/*@*/
				if(aMessage == document.getElementById("bundle_quickreplyMessages").getString('copyCompleteHack') ) { // XX dirty hack! Otherwise, the message lingers. -- hard to localise, too
					window.setTimeout("setStatus('');clearQR();enableQR();", "3000");
				}
			} catch (e) {
				debug(5,'onStatusChange: exception "'+e+'" caught when getting string copyCompleteHack');
			};
		},
    
        onSecurityChange: function(aWebProgress, aRequest, state)
        {
          // we can ignore this notification
        },
    
        QueryInterface : function(iid)
        {
          if (iid.equals(Components.interfaces.nsIWebProgressListener) ||
              iid.equals(Components.interfaces.nsISupportsWeakReference) ||
              iid.equals(Components.interfaces.nsISupports))
            return this;
         
          throw Components.results.NS_NOINTERFACE;
        }
    };

    // alert any observers that may be looking for a composeOnSend event
    var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    observerService.notifyObservers(window, "mail:composeOnSend", null);
    
	var progress = Components.classes["@mozilla.org/messenger/progress;1"].createInstance(Components.interfaces.nsIMsgProgress);
	if(!progress) {
		return doReplyFailed(document
				.getElementById("bundle_quickreplyMessages")
				.getString('failInitProgress'),
				gTypedBdy );
				//"Failed to initialise progress for quick reply."
	}
	progress.registerListener(progressListener);
	progressListener.srcMsgURI = gSrcMsgURI;

    // set up a fallback character set should the message text need it
    var fallbackCharset = new Object;
    gPromptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    if (gPromptService && !msgCompose.checkCharsetConversion(getCurrentIdentity(), fallbackCharset)) {
        var dlgTitle = document.getElementById("bundle_quickreplyMessages").getString('initErrorDlogTitle');
        var dlgText = document.getElementById("bundle_quickreplyMessages").getString('12553'); // NS_ERROR_MSG_MULTILINGUAL_SEND
        if (!gPromptService.confirm(window, dlgTitle, dlgText)) 
            {}
        else {
            fallbackCharset.value = "UTF-8";
    		    debug(5, 'doReply: in fallbackCharset set to UTF-8');
        }
    }
    if (fallbackCharset && fallbackCharset.value && fallbackCharset.value != "")
        msgCompose.SetDocumentCharset(fallbackCharset.value);
   
    gPromptService = null;
 
    // just before we try to send the message, fire off the compose-send-message event for listeners
    // such as smime so they can do any pre-security work such as fetching certificates before sending

    // !!! this is a hard one to emulate, cannot mimic it completely because it's well
    // tied in to the real Compose window.. looks like encryption may not work right because its not fully set up 
    // this is commented out for the time being until I can figure out how to make it work
    //var event = document.createEvent('Events');
    //event.initEvent('compose-send-message', false, true);
    //document.getElementById("messengerWindow").dispatchEvent(event);


    
    // Lets get the account for this folder, needed for SendMsg - Frank DiLecce
    var account =  gAccountManager.FindAccountForServer(GetLoadedMsgFolder().server);
    // The default compose window sendmsg function has it, needed for SendMsg - Frank DiLecce
	debug(5, 'doSend: account key to use ' + account.key);
    msgWindow.SetDOMWindow(window);
    msgCompose.editor.QueryInterface(nsIPlaintextEditorMail).wrapWidth = msgCompose.wrapLength;
    GetCurrentEditor().QueryInterface(Components.interfaces.nsIEditorMailSupport).rewrap(true);    
	try {
		if( gSendNow ) {
			debug(5, 'doSend: initiating send now');
            // next line changed to match SendMsg parameter needs - Frank DiLecce
			//msgCompose.SendMsg(nsIMsgCompDeliverMode.Now, identity, progress);
			msgCompose.SendMsg(nsIMsgCompDeliverMode.Now, identity, account.key, msgWindow, progress);

    	} else {
			debug(5, 'doSend: initiating send later');
            // next line changed to match SendMsg parameter needs - Frank DiLecce
			//msgCompose.SendMsg(nsIMsgCompDeliverMode.Later, identity, progress);
			msgCompose.SendMsg(nsIMsgCompDeliverMode.Later, identity, account.key, msgWindow, progress);
		}
	} catch(e) {
        gQREditorWindow.close();
        alert(e);
		debug(5, e);
		return doReplyFailed(document.getElementById("bundle_quickreplyMessages").getString('failMsgSend') + " >>" + e,	gTypedBdy);
        msgCompose = null;
        GetCurrentEditorElement().commandManager.removeCommandObserver(gMsgEditorCreationObserver, "obs_documentCreated");
        GetCurrentEditorElement().makeEditable("textmail", false);
	}
}
