/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 1998-1999 Netscape Communications Corporation. All
 * Rights Reserved.
 */

// This is where functions related to the quick reply box live.

// interfaces
var nsIMsgCompDeliverMode = Components.interfaces.nsIMsgCompDeliverMode;
var nsIMsgCompSendFormat = Components.interfaces.nsIMsgCompSendFormat;
var nsIMsgCompConvertible = Components.interfaces.nsIMsgCompConvertible;
var nsIMsgCompType = Components.interfaces.nsIMsgCompType;
var nsIMsgCompFormat = Components.interfaces.nsIMsgCompFormat;
var nsIAbPreferMailFormat = Components.interfaces.nsIAbPreferMailFormat;
var nsIPlaintextEditorMail = Components.interfaces.nsIPlaintextEditor;

// static globals, need to be initialized only once
var sMsgComposeService = Components.classes["@mozilla.org/messengercompose;1"].getService(Components.interfaces.nsIMsgComposeService);
var sConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
var gAccountManager = Components.classes["@mozilla.org/messenger/account-manager;1"].getService(Components.interfaces.nsIMsgAccountManager);

var sPrefs = null;
var sPrefBranchInternal = null;
var sOther_headers = "";

var sAccountManagerDataSource = null;
var sRDF = null;
var sNameProperty = null;

var sQuickReplyMessagesBundle = document.getElementById("bundle_quickreplyMessages");
var sComposeMessagesBundle = document.getElementById("bundle_composeMessages");

var gDebugLevel;
try {
	gDebugLevel = pref.getIntPref( 'quickreply.debug' );
} catch(e) {
    saveDefaultPrefs();
	gDebugLevel = 0;
}

// debug function.
// the higher level, the less chance it gets shown.
// the lower gDebugLevel, the less gets shown.
// debug levels:
//	5:	important information is logged, including catches.
//	10:	non-default paths are logged
// 	20:	all function starts get shown
//	25:	all key presses in text box are also shown.
 function debug(level, message) {
	if( level > gDebugLevel ) { return; }
	 sConsoleService.logStringMessage('QR: '+message);
}

//function ShowPref(tgt) { alert('boe'); }

// shortcut key for focus
function setFocusToQRTextInput(tgt) {
	debug(20, 'setFocusToQRTextInput: started');
	var viewQuickReplyToolbar;
	try {
			viewQuickReplyToolbar = pref.getBoolPref( 'quickreply.display_' + tgt );
	} catch (e) {
			debug(5,'setFocusToQRTextInput: exception "'+e+'" caught when accessing preference quickreply.display_' + tgt);
			viewQuickReplyToolbar = 1;
	}
	if(viewQuickReplyToolbar) {
		document.getElementById('quickReplyInput').focus();
	}
}

// getActiveQuoteChoice
// depending on preferences, returns 0, 1 or 2. 0: don't quote, 1: above, 2: below
function getActiveQuoteChoice() {
	debug(10, 'getActiveQuoteChoice: started');
	var shouldQuotePref, retval;
	// shouldQuotePref:
	// 0: use system pref, 1: don't, 2: above, 3: below
	try {
			shouldQuotePref = pref.getIntPref('quickreply.quoting');
	} catch (e) {
		debug(5,'getActiveQuoteChoice: exception "'+e+'" caught when accessing preference quickreply.quoting');
			shouldQuotePref = 0;
			pref.setIntPref( 'quickreply.quoting', shouldQuotePref );
	}
	
	if( shouldQuotePref == 0 ) {
	
		// mail.autoqoute defaults to true!
		retval = -1;
		try {
			// should we quote or not?
			if( false == pref.getBoolPref("mail.identity." + getCurrentIdentity().key + ".auto_quote") ) {
				retval = 0;
			}
		} catch(e) {
			debug(5,'getActiveQuoteChoice: exception "'+e+'" caught when accessing preference mail.autoquote');
		}
		// we should quote, maybe. now, let's find out where.
		if( retval == -1 ) {
			try {
				// if the pref is there, it's either 1 (below) or 2 (select text)
				switch( pref.getIntPref("mail.identity." + getCurrentIdentity().key + ".reply_on_top") ) {
					case 1: retval = 1; break;	// above
					case 2: retval = 2; break;// select - interpret as below (we need to do something...)
					default: retval = 2;
				}
			} catch(e) {
				// if the getting fails, the pref is not there - and then we need to quote below.
				debug(5,'getActiveQuoteChoice: exception "'+e+'" caught when accessing preference mailnews.reply_on_top');
				retval = 2;
			}
		}
	} else {
		retval = shouldQuotePref - 1;
	}
	return retval;
}

// getActiveSigChoice
// depending on preferences, returns 0 or 1. 0: don't, 1: bottom, 2: below.
function getActiveSigChoice( identity ) {
	debug(20, 'getActiveSigChoice: started');
	var retval, sigpref;
	// quickreply.signature:
	//   0: Honour the account's signature preference
	//   1: Don't add a signature (default)
	//   2: Add the signature at the bottom of the email
	//   3: Add the signature directly below my text
	try {
		sigpref = pref.getIntPref('quickreply.signature');
	} catch (e) {
		debug(5,'getActiveSigChoice: exception "'+e+'" caught when accessing preference quickreply.signature');
		sigpref = 1;
		pref.setBoolPref( 'quickreply.signature', sigpref );
	}
	if( sigpref == 0 ) {
		// if the pref says FALSE, don't add it (0)
		// if the pref says TRUE,  add it as follows:
		// if identity.sigBottom is not defined, then add at bottom (1)
		// if identity.sigBottom is defined and true, then add at bottom (1)
		// if identity.sigBottom is defined and false, then add directly below my text (2)
		if( identity.attachSignature ) {
			if( identity.sigBottom === undefined ) { // sigBottom is not defined - possibly have an old TB.
				retval = 1;
			} else {
				retval = ( identity.sigBottom ? 1 : 2 );
			}
		} else {
			retval = 0;
		}
	} else {
		retval = sigpref - 1; // what a coincidence...
	}
	
	return retval;
}

// menu item
function initQRMenuItem(tgt) {
	debug(20, 'initQRMenuItem: started');
	var viewQuickReplyToolbar;
	try {
			viewQuickReplyToolbar = pref.getBoolPref('quickreply.display_' + tgt);
	} catch (e) {
			debug(5,'initQRMenuItem: exception "'+e+'" caught when accessing preference quickreply.display_' + tgt);
			viewQuickReplyToolbar = 1;
			pref.setBoolPref( 'quickreply.display_' + tgt, viewQuickReplyToolbar );
	}
	document.getElementById('menu_showQuickReplyToolbar').setAttribute("checked", viewQuickReplyToolbar ? "true" : "false");
}

function saveDefaultPrefs() {
	debug(20, 'saveDefaultPrefs: started');
	pref.setIntPref("quickreply.debug", 0);
	pref.setBoolPref("quickreply.deleteaftersend", true);
	pref.setBoolPref("quickreply.display_mailWindow", false);
	pref.setBoolPref("quickreply.display_toggle_mailWindow", false);
	pref.setIntPref("quickreply.ident", 1);
	pref.setIntPref("quickreply.quoting", 0);
	pref.setIntPref("quickreply.signature", 0);
}

function getViewQuickReplyToolbar(tgt) {
	debug(20, 'getViewQuickReplyToolbar: started');
	try {
			viewQuickReplyToolbar = pref.getBoolPref( 'quickreply.display_' + tgt );
	} catch (e) {
			debug(5,'getViewQuickReplyToolbar: exception "'+e+'" caught when accessing preference quickreply.display_' + tgt );
// added by Frank DiLecce
// can only assume NO prefs are set up .. lets try to set up defaults see
            saveDefaultPrefs();
			viewQuickReplyToolbar = 0;
	}
	try {
			viewQuickReplyToolbarToggle = pref.getBoolPref( 'quickreply.display_toggle_' + tgt );
	} catch (e) {
			debug(5,'getViewQuickReplyToolbar: exception "'+e+'" caught when accessing preference quickreply.display_toggle_' + tgt );
			viewQuickReplyToolbarToggle = 0;
	}
	if (viewQuickReplyToolbarToggle)
		return 2;
	return viewQuickReplyToolbar;
}

function setViewQuickReplyToolbar(tgt, viewQuickReplyToolbar) {
	debug(20, 'setViewQuickReplyToolbar: started');
	if(viewQuickReplyToolbar) {
			document.getElementById('menu_showQuickReplyToolbar').setAttribute("checked", "true");
			ShowQuickReplyBox(tgt);
	} else {
			document.getElementById('menu_showQuickReplyToolbar').setAttribute("checked", "false");
			HideQuickReplyBox(tgt);
	}
	pref.setBoolPref( 'quickreply.display_toggle_' + tgt, viewQuickReplyToolbar == 2 );
	pref.setBoolPref( 'quickreply.display_' + tgt, viewQuickReplyToolbar );
}

function toggleQuickReplyMenuItem(tgt) {
	debug(20, 'toggleQuickReplyMenuItem: started');
	var viewQuickReplyToolbar = getViewQuickReplyToolbar(tgt);
	if (viewQuickReplyToolbar == 0)
		viewQuickReplyToolbar = 1;
	else
		viewQuickReplyToolbar = 0;
	setViewQuickReplyToolbar(tgt, viewQuickReplyToolbar);
}

function toggleQuickReplyButton(tgt) {
	debug(20, 'toggleQuickReplyButton: started');
	var viewQuickReplyToolbar = getViewQuickReplyToolbar(tgt);
	if (viewQuickReplyToolbar == 0)
		viewQuickReplyToolbar = 2;
	else
		viewQuickReplyToolbar = 0;
	setViewQuickReplyToolbar(tgt, viewQuickReplyToolbar);
}

//Stuff connected to the interface
function getCurrentIdentity() {
	debug(20, 'getCurrentIdentity: started');
	var server = GetLoadedMsgFolder().server;
	var identity = accountManager.getFirstIdentityForServer( server );
	if( ! identity ) {
		var account = accountManager.defaultAccount;
		identity = account.defaultIdentity;
		debug(10, 'Using default identity');
    	if( ! identity ) {
            // if identity is STILL blank, use the one from prefs
            identity = accountManager.getIdentity("id" + pref.getIntPref( 'quickreply.ident' ));
    		debug(10, 'Default identity was blank, using pref Identity');
        }
	}
	debug(10, 'Identity: '+identity);
	return identity;
}


function setStatus(newStatus) {
	document.getElementById("statusText").setAttribute("label", newStatus);
}

function ShowQuickReplyBox(tgt) {
	debug(20, 'ShowQuickReplyBox: started');
	var QRBox = document.getElementById( 'msgQuickReply_' + tgt);
	if (QRBox)
			QRBox.collapsed = false;
} 

function HideQuickReplyBox(tgt) { 
	debug(20, 'HideQuickReplyBox: started');
	var QRBox = document.getElementById( 'msgQuickReply_' + tgt);
	if (QRBox)
			QRBox.collapsed = true;
}


function ShowPref(tgt) {
	debug(20, 'ShowPref: started');
	window.openDialog('chrome://messenger-quickreply/content/prefQuickReply.xul', 'messenger-quickreply-pref', 'chrome');
}

function disableQR() {
	debug(20, 'disableQR: started');
	document.getElementById("sendButton").disabled=true;
	document.getElementById("quickReplyInput").disabled=true;
}

function enableQR() {
	debug(20, 'enableQR: started');
	document.getElementById("sendButton").disabled=false;
	document.getElementById("quickReplyInput").disabled=false;
}

function clearQR() {
	debug(20, 'clearQR: started');
	try {
		if( pref.getBoolPref( 'quickreply.deleteaftersend') )
			document.getElementById("quickReplyInput").value="";
	} catch(e) {
		debug(5,'clearQR: exception "'+e+'" caught when accessing preference quickreply.deleteaftersend');
		document.getElementById("quickReplyInput").value="";
	}
}
function doQRTextBoxFocus(e) {
	debug(20, 'doQRTextBoxFocus: started');
//	var tb = document.getElementById('quickReplyInput');
	setStatus('tb focus '+e.type);
}

function doQRTextBoxClick(e) {
	debug(20, 'doQRTextBoxClick: started');
//	var tb = document.getElementById('quickReplyInput');
//	if(!tb.focussed) tb.select();
}

function doQRTextBoxKeyPress(e, tgt) {
	debug(25, 'doQRTextBoxKeyPress: '+e.type+"|ctrl:"+e.ctrlKey+"|alt:"+e.altKey+"|keyCode:"+e.keyCode+"|charCode:"+e.charCode+"|which:"+e.which);
	if(e.ctrlKey == true && e.altKey == false && e.metaKey == false && e.which== 13 ) { // Enter: 13
		// ctrl+enter --> send; check if that's ok.
		disableQR();
		if( !maybeWarnOnSend() ) {
			enableQR();
			return;
		}
		doQuickReply(e, -1, tgt);	// just pass the same event - saves the hassle.
	} else if(e.ctrlKey == false && e.altKey == false && e.metaKey == false && e.keyCode == 9) {
		// tab: insert tab character.
		e.preventDefault();
		var textbox = document.getElementById("quickReplyInput")
		var content = textbox.value;
		var selStart = textbox.selectionStart;
		var selEnd = textbox.selectionEnd;
		textbox.value = content.substr(0, selStart )+"\t"+content.substr(selEnd, content.length );
		textbox.setSelectionRange( selStart+1, selStart+1);
	}
}

// function getQuotedOriginal
//	This returns the original message (or a selected part) quoted for inclusion in the reply
function getQuotedOriginal( author ) {
	debug(20, 'getQuotedOriginal: started');
	// try to get selection from the message pane
	var composeMsgsBundle = document.getElementById("bundle_composeMessages");
	
	var messagepaneframe = GetMessagePaneFrame();
	var selection = messagepaneframe.getSelection();
	var selectiontext = selection.toString();
	if (selectiontext == "") {
		// if no selection, quote the whole document...
		selection.selectAllChildren(messagepaneframe.document.body);
		selectiontext = selection.toString();
		selection.collapseToStart();
	}
	// say who wrote the quoted part
	var authorwrote;
	try {
		authorwrote = composeMsgsBundle.getString('mailnews.reply_header_authorwrote')
	} catch(e) {
		debug(5,'getQuotedOriginal: exception "'+e+'" caught when accessing message mailnews.reply_header_authorwrote' );
		authorwrote = '';
	}

	// var quoteauthor = author + " wrote:";
	var quoteauthor;
	if( authorwrote == '' ) {
		quoteauthor = author;
	} else {
	 	quoteauthor = authorwrote.replace( '%s', author ) + ':';
	}

	// Add an indentCharacter at the beginning of each line.
	// The indentCharacter should be localised in the main app, but I can't find where - pending that, it's in our
	// personal properties. The indentCharacter defaults to '> '.
	var indentCharacter;
	try {
		/*@*/
		indentCharacter = document.getElementById("bundle_quickreplyMessages").getString('indentCharacter');
	} catch (e) {
		debug(5,'getViewQuickReplyToolbar: exception "'+e+'" caught when getting string indentCharacter' );
		indentCharacter = '> ';
	}

        var quotedsection = "";
        var eolpos = 0;
        var endofline = new RegExp("$", "m");
        var stilltoreplace = selectiontext;
        while (stilltoreplace != "") {
		eolpos = stilltoreplace.search(endofline);
		if (eolpos >= 0) {
			quotedsection = quotedsection + indentCharacter + stilltoreplace.slice(0, eolpos+1);
			stilltoreplace = stilltoreplace.slice(eolpos+2);//1);
		} else {
			quotedsection = quotedsection + indentCharacter + stilltoreplace;
		}
        }
	// put them all together...
	return quoteauthor + "\n" + quotedsection + "\n";
}

// function doQuickReply
//	This is the callback of the QuickReply button.
//	e: event that caused this click.
//	quote: if the original message should be quoted or not. -1 = use default value, 0 = don't, 1=above, 2=below
function doQuickReply(e, quote, tgt) {
	debug(10, 'doQuickReply: started');
    // just gonna test to see if any prefs are set
	try {
			qvtemp = pref.getBoolPref( 'quickreply.display_toggle_' + tgt );
	} catch (e) {
    // added by Frank DiLecce
    // can only assume NO prefs are set up .. lets try to set up defaults see
    }
	if (document.getElementById("quickReplyInput").value == null ||
			document.getElementById("quickReplyInput").value == "")	{
			debug(5, 'doQuickReply: nothing to reply to - returning');
			return;
	}
	
	var srcMsgHdr, dest, newsgroups, author, bdy, typedbdy, identity;
	disableQR();
	var srcMsgURI = GetLoadedMessage();
	try {
			srcMsgHdr = messenger.messageServiceFromURI(srcMsgURI).messageURIToMsgHdr(srcMsgURI)//GetMsgHdrFromUri(srcMsgURI);
	} catch (ex) {
			debug(5,'doQuickReply: exception "'+e+'" caught when trying to get message header for URI '+srcMsgURI+'.' );
			enableQR();
			return;
	}
	var srcMsgId = srcMsgHdr.messageId;
	var subj = srcMsgHdr.subject; //currentHeaderData["subject"].headerValue;

	// dwlt: is this localized in the main Mozilla app? If so, is there a way to request this value?
	//		::can't find it in the main components anywhere, but haven't looked in the C++ source base
	//		::if we can't find it there, we should just put in the quickreply.properties file
	
	// replySubjectPrefix is normally 'Re: '
	var replySubjectPrefix;
	try {
		/*@*/
		replySubjectPrefix = document.getElementById("bundle_quickreplyMessages").getString('replySubjectPrefix');
	} catch (e) {
		debug(5,'doQuickReply: exception "'+e+'" caught when getting string replySubjectPrefix' );
		replySubjectPrefix = 'Re: ';
	}
	debug(15,'doQuickReply: replySubjectPrefix: "'+replySubjectPrefix+'".' );
	
	if( subj.substring(0,4).toLowerCase() != replySubjectPrefix.toLowerCase() ) {
			subj = replySubjectPrefix + subj;
	}

	dest = "";
	newsgroups = "";
	if ('newsgroups' in currentHeaderData) {
		if ('followupto' in currentHeaderData)
			newsgroups = currentHeaderData['followupto'].headerValue;
		else
			newsgroups = currentHeaderData['newsgroups'].headerValue;
		debug(10, 'doQuickReply: replying to news groups: '+newsgroups);
	} else {
		if ('reply-to' in currentHeaderData)
			dest = currentHeaderData['reply-to'].headerValue;
	else
			dest = srcMsgHdr.author;
		debug(10, 'doQuickReply: replying to email: '+dest);
	}
	
	var identity = getCurrentIdentity();
	if( ! identity ) {
		doReplyFailed( document
					.getElementById("bundle_quickreplyMessages")
					.getString('failCurrentIdentity'),
					'' );
	}
			
	var shouldSig = getActiveSigChoice( identity );
	debug(15,'doQuickReply: shouldSig: "'+shouldSig+'".' );
	var sigContent = '';
	if( shouldSig > 0 ) {
		var sigFile = identity.signature;
		if (sigFile) {
			try{
				//var obj_File = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
				//obj_File.initWithPath(str_Filename);
	
				var sigInputStream =  Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
				sigInputStream.init( sigFile, 0x01, 0444, null );
				
				var sigScriptableIO = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
				sigScriptableIO.init(sigInputStream);
			} catch (e) {
				debug(5,'doQuickReply: exception "'+e+'" caught when accessing sig file "'+sigFile.persistentDescriptor+'". Does the file exist?');
			}

			try {
				sigContent = sigScriptableIO.read( sigFile.fileSize );
			} catch (e) {
				sigContent = '';
				debug(5,'doQuickReply: exception "'+e+'" caught when reading sig file "'+sigFile.persistentDescriptor+'". Does the file exist?');
			}

			try {
			sigScriptableIO.close();
			sigInputStream.close();
			} catch (e) { //don't report - not of consequence, really.
			}
		} else {
			debug(5,'doQuickReply: signature required but not defined for identity '+identity);
		}
	}

	var shouldQuote, defQuotePref = 0;
	
	bdy = document.getElementById("quickReplyInput").value;
	typedbdy = bdy;
	// do we need to quote?
	if( quote != 0 ) {
		// use the standard preference
		if( quote < 0 ) {
			defQuotePref = getActiveQuoteChoice();
		} else {
			switch( quote ) {
				// 1: above, 2: below
				case 1: case 2: defQuotePref = quote; break;
				default: defQuotePref = 0;
			}
		}
		debug(15,'doQuickReply: defQuotePref: "'+defQuotePref+'".' );
		// add sig below - so add to bdy - no matter what happens,
		//  it'll be in the right spot
		if ( shouldSig == 2 ) {
			bdy = bdy + "\n" + sigContent + "\n";
		}
		if( defQuotePref > 0 ) {
			var orig = getQuotedOriginal(srcMsgHdr.author);
			if( defQuotePref == 1 ) {
				bdy = bdy + "\n\n" + orig;
			} else {
				bdy = orig + "\n" + bdy;
			}
		}
		// add sig at the bottom.
		if( shouldSig == 1 ) {
			bdy += "\n" + sigContent + "\n";
		}
	} else {
		// don't care about whether it's bottom or below - amounts to same anyway.
		if( shouldSig > 0 ) {
			bdy += "\n" + sigContent + "\n";
		}
	}
	doReply( e.shiftKey == false, dest, newsgroups, subj, typedbdy, srcMsgId, srcMsgURI, bdy, identity );
	// hide the quickReply bar if it was opened by the toggle button
	if (getViewQuickReplyToolbar(tgt) == 2)
		setViewQuickReplyToolbar(tgt, 0);
}

// Check mail.warn_on_send_accel_key, and if necessary, ask for confirmation.
// Returns true if it's OK to send, false if not.
function maybeWarnOnSend() {
	debug(20, 'maybeWarnOnSend: started');
    var warn = pref.getBoolPref("mail.warn_on_send_accel_key");
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    var composeMsgsBundle = document.getElementById("bundle_composeMessages");

    if (warn) {
        var checkValue = {value:false};
        var buttonPressed = promptService.confirmEx(window, 
              composeMsgsBundle.getString('sendMessageCheckWindowTitle'), 
              composeMsgsBundle.getString('sendMessageCheckLabel'),
              (promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_0) +
              (promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_1),
              composeMsgsBundle.getString('sendMessageCheckSendButtonLabel'),
              null, null,
              composeMsgsBundle.getString('CheckMsg'), 
              checkValue);
        if (buttonPressed != 0) {
        	debug(10, 'maybeWarnOnSend: sending cancelled.');
            return false;
        }
        if (checkValue.value) {
            pref.setBoolPref("mail.warn_on_send_accel_key", false);
        }
    }
    return true;
}

// Under-the-hood stuff
function onSMIMEStartHeadersQuickReply() {
}

function onSMIMEEndHeadersQuickReply() {
	ShowQuickReplyBox();
	document.getElementById("QuickReplyTarget").value = currentHeaderData["to"].headerValue ;
}

function regListeners() {
	var listener = {};
	listener.onStartHeaders = onSMIMEStartHeadersQuickReply;
	listener.onEndHeaders = onSMIMEEndHeadersQuickReply;
	gMessageListeners.push(listener);
}

function QRMessagePaneOnLoad() {
// window.setTimeout("regListeners()", 1000);
	regListeners();
}

function QRMessagePaneOnUnload() {
}

addEventListener("messagepane-loaded", QRMessagePaneOnLoad, true);
addEventListener("messagepane-unloaded", QRMessagePaneOnUnload, true);

function markMessageReplied( msgURI) {
	debug(20, 'markMessageReplied: started');
	var msgHdr = messenger.messageServiceFromURI(msgURI).messageURIToMsgHdr(msgURI)//GetMsgHdrFromUri(srcMsgURI);
	if( !msgHdr) {
		return;
	}
	var msgFolder = msgHdr.folder;
	if (msgFolder) {
		msgFolder.addMessageDispositionState(msgHdr, 0);//msgFolder.nsMsgDispositionState_Replied
	}
}

function doReplyFailed( why, content ) {
	debug(5, 'Quick Reply failed: '+why);

	if(why) { dump(why); }
	setStatus( why );
	setTimeout( "setStatus('')", 10000 );
	if( content.length > 0 ) {
		document.getElementById("quickReplyInput").value = content;
	}
	enableQR();
	return false;
}

function queryISupportsArray(supportsArray, iid) {
// this function required in order to get an array of accounts for use in doReply : added by Frank DiLecce
    var result = new Array;
    for (var i=0; i<supportsArray.Count(); i++) {
      // dump(i + "," + result[i] + "\n");
      result[i] = supportsArray.QueryElementAt(i, iid);
    }
    return result;
}


// the actual sending function.
function doReply( sendNow, to, newsgroups, sbj, typedbdy, srcId, srcMsgURI, bdy, identity ) {
	debug(10, 'doReply: started');
	// all progress notifications are done through the nsIWebProgressListener implementation...
	var progressListener = {
		srcMessageId: String,
		onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
			debug(10, 'onStateChange: '+aStatus);
			if (aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_START) {
	//			setStatus( sQuickReplyMessagesBundle.getString('statusStartingSend') );
				setStatus( document.getElementById("bundle_quickreplyMessages").getString('statusStartingSend') );
			}
			if (aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_STOP) {
				clearQR();
				enableQR();
	//			setStatus( sQuickReplyMessagesBundle.getString('statusMessageSent') );
				setStatus( document.getElementById("bundle_quickreplyMessages").getString('statusMessageSent') );
				window.setTimeout("setStatus('')", "3000");
				markMessageReplied( this.srcMsgURI );
				debug(5, 'onStateChange: message sent');
			}
		},
		
		// could implement progress bar using aCurTotalProgress/aMaxTotalProgress
		onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {
			var perc = Math.round(aCurTotalProgress); ///aMaxTotalProgress);
			debug(20, 'onProgressChange:, '+perc+ " " + aCurSelfProgress+ " " +aMaxTotalProgress+ " " +aWebProgress);
	//			setStatus( sQuickReplyMessagesBundle.getString('statusMessageSending')+" "+perc+"%");
					setStatus( document.getElementById("bundle_quickreplyMessages").getString('statusMessageSending') + '    ' + perc + '%' );
		},
	
		onLocationChange: function(aWebProgress, aRequest, aLocation)
		{},
	
		onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) {
			// Looks like it's possible that we get call while the document has been already delete!
			// therefore we need to protect ourself by using try/catch
			debug(10, 'onStatusChange: '+aStatus+':'+aMessage+'.');
			try {
				setStatus( aMessage );
				/*@*/
				if(aMessage == document.getElementById("bundle_quickreplyMessages").getString('copyCompleteHack') ) { // XX dirty hack! Otherwise, the message lingers. -- hard to localise, too
					window.setTimeout("setStatus('');clearQR();enableQR();", "3000");
				}
			} catch (e) {
				debug(5,'onStatusChange: exception "'+e+'" caught when getting string copyCompleteHack');
			};
		},
	
		onSecurityChange: function(aWebProgress, aRequest, state)
		{},
	
		QueryInterface : function(iid)
		{
				if (iid.equals(Components.interfaces.nsIWebProgressListener) ||
							iid.equals(Components.interfaces.nsISupportsWeakReference) ||
							iid.equals(Components.interfaces.nsISupports))
					return this;
				
				throw Components.results.NS_NOINTERFACE;
		}
	};

	// set references & in-reply-to
	var compfields = Components.classes["@mozilla.org/messengercompose/composefields;1"].createInstance(Components.interfaces.nsIMsgCompFields);
	if(!compfields) {
//			return doReplyFailed(sQuickReplyMessagesBundle.getString('failCompFields') ); //"Failed to initialise compfields for quick reply."
				return doReplyFailed(document
						.getElementById("bundle_quickreplyMessages")
						.getString('failCompFields'),
						typedbdy);
						//"Failed to initialise compfields for quick reply."
	}
	var msgCompose = Components.classes["@mozilla.org/messengercompose/compose;1"].createInstance(Components.interfaces.nsIMsgCompose);
	if(!msgCompose) {
//			return doReplyFailed(sQuickReplyMessagesBundle.getString('failMsgCompose') "Failed to initialise msgCompose for quick reply.");
				return doReplyFailed(document.getElementById("bundle_quickreplyMessages").getString('failMsgCompose') );//"Failed to initialise compfields for quick reply."
	}
	var params = Components.classes["@mozilla.org/messengercompose/composeparams;1"].createInstance(Components.interfaces.nsIMsgComposeParams);
	if(!params) {
//			return doReplyFailed(sQuickReplyMessagesBundle.getString('failInitParams') ); //"Failed to initialise params for quick reply."
				return doReplyFailed(document
						.getElementById("bundle_quickreplyMessages")
						.getString('failInitParams'),
						typedbdy);
						//"Failed to initialise compfields for quick reply."
	}
	var progress = Components.classes["@mozilla.org/messenger/progress;1"].createInstance(Components.interfaces.nsIMsgProgress);
	if(!progress) {
//			return doReplyFailed(sQuickReplyMessagesBundle.getString('failInitProgress') ); //"Failed to initialise progress for quick reply."
				return doReplyFailed(document
						.getElementById("bundle_quickreplyMessages")
						.getString('failInitProgress'),
						typedbdy );
						//"Failed to initialise compfields for quick reply."
	}
	progress.registerListener(progressListener);
	progressListener.srcMsgURI = srcMsgURI;
/**/
	if (to != "")
	compfields.to = to;
	if (newsgroups != "")
		compfields.newsgroups = newsgroups;
	compfields.subject = sbj;
	compfields.body = bdy ;
	compfields.references = "<"+srcId+">";
	
	debug(15, 'doReply: to: '+to);
	debug(15, 'doReply: newsgroups: '+newsgroups);

	params.composeFields = compfields;
	params.format = 2;	// PlainText, otherwise we loose our formatting!
	msgCompose.Initialize(null, params );


// these are the accounts for all of Moz  ; added by Frank DiLecce
    var accounts = queryISupportsArray(gAccountManager.accounts, Components.interfaces.nsIMsgAccount);
// we only need the current one,  the SendMsg function needs the account key
    var account =  gAccountManager.FindAccountForServer(GetLoadedMsgFolder().server);
// this might not be needed but the real compose has it  : added by Frank DiLecce
    msgWindow.SetDOMWindow(window);

	try {
		if( sendNow ) {
			debug(5, 'doReply: initiating send now');
			msgCompose.SendMsg(nsIMsgCompDeliverMode.Now, identity, account.key, msgWindow, progress);
    	} else {
			debug(5, 'doReply: initiating send later');
			msgCompose.SendMsg(nsIMsgCompDeliverMode.Later, identity, account.key, msgWindow, progress);
		}
	} catch(e) {
		return doReplyFailed(document
				.getElementById("bundle_quickreplyMessages")
				.getString('failMsgSend'),
				typedbdy);
	}
}
